# Generated by roxygen2: do not edit by hand

S3method(print,osrm_server_list)
export(osrm_check_available_versions)
export(osrm_check_latest_version)
export(osrm_cleanup)
export(osrm_clear_path)
export(osrm_contract)
export(osrm_customize)
export(osrm_extract)
export(osrm_find_profile)
export(osrm_get_server_profile)
export(osrm_gui)
export(osrm_install)
export(osrm_partition)
export(osrm_prepare_graph)
export(osrm_servers)
export(osrm_start)
export(osrm_start_server)
export(osrm_stop)
export(osrm_stop_all)
export(osrm_uninstall)
export(osrm_which)
importFrom(lifecycle,deprecated)
