% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/osrm_server_registry.R
\name{osrm_get_server_profile}
\alias{osrm_get_server_profile}
\title{Retrieve the OSRM Profile for a Running Server}
\usage{
osrm_get_server_profile(input_osrm = NULL, port = NULL)
}
\arguments{
\item{input_osrm}{Optional. Can be an OSRM job process (an \code{osrm_server}
object inheriting from \code{processx::process}), a path string, or NULL.}

\item{port}{Optional integer. The port of the server.}
}
\value{
A character string representing the profile name (default "car").
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

Attempts to determine the profile (e.g., "car", "bike", "foot") used by an OSRM server.
It follows a priority list:
\enumerate{
\item Checks the active server registry for the given port or ID.
\item Checks for a \code{dataset.meta.json} file in the directory of the graph file.
\item Checks the graph filename for hints (e.g. \code{berlin-car.osrm}).
\item Falls back to \code{getOption("osrm.profile")}.
}
}
