% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/osrm_install.R
\name{osrm_check_latest_version}
\alias{osrm_check_latest_version}
\title{Check for the Latest Stable OSRM Version}
\usage{
osrm_check_latest_version()
}
\value{
A string containing the latest version tag (e.g., \code{"v26.4.1"}).
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

Queries the GitHub API to find the most recent stable (non-pre-release)
version tag for the OSRM backend that has binaries available for the current platform.
}
\examples{
\donttest{
if (identical(Sys.getenv("OSRM_EXAMPLES"), "true")) {
  # Get the latest stable version number of OSRM backend
  osrm_check_latest_version()
}
}
}
